@@ -51,7 +51,7 @@ class TestExecuteStorageCapacityBalanceStrategy(
         self.create_volume(imageRef=CONF.compute.image_ref, size=3)
         self.create_volume(imageRef=CONF.compute.image_ref, size=3)
         self.create_volume(imageRef=CONF.compute.image_ref, size=3)
-        self.create_volume(imageRef=CONF.compute.image_ref)
+        self.create_volume(imageRef=CONF.compute.image_ref, size=1)
         audit_parameters = {"volume_threshold": 25}
 
         _, goal = self.client.show_goal(self.GOAL)
